#ifndef PROBLEM_H
#define PROBLEM_H

#include "domainrighthandside.h"
#include "dirichletdata.h"
#include "problemdescriptorbase.h"
#include "myequation.h"


using namespace Gascoigne;



// class for specifying problem's right hand side
class MyRhs : public DomainRightHandSide
{
  MyData data;

  public:

  MyRhs(const MyData& dat) : data(dat)
  {
  }

  MyRhs* createNew() const
  {
      return new MyRhs(data);
  }

  int GetNcomp() const { return 1; }
  std::string GetName() const { return "Right Hand Side"; }

  double operator()(int c, const Vertex2d &v) const
  {
    return 0.;
  }
};


// class for specifying Dirichlet data
class MyDirichletData : public DirichletData
{
public:
  MyDirichletData(const ParamFile & pf) : DirichletData(pf)
  {}
  std::string GetName() const
  {
    return "Dirichlet Data";
  }

  void operator()(DoubleVector &b, const Vertex2d &v, int col) const
  {
    // TO DO: Implement Dirichlet data
    b[0] = 0;
  }
};



class ProblemDescriptor : public ProblemDescriptorBase {

  MyData data;

public:

  std::string GetName() const  { return "ConvDiff";}


  void BasicInit(const ParamFile & pf)
  {
    GetParamFile() = pf;
    data.BasicInit(pf);

    // equation to solve
    GetEquationPointer() = new MyEquation(data);

    // definition for right hand side
    GetRightHandSidePointer() = new MyRhs(data);

    // definition of dirichlet boundary data
    GetDirichletDataPointer() = new MyDirichletData(pf);

    ProblemDescriptorBase::BasicInit(pf);
  }
};


#endif /* PROBLEM_H */
